﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;                 //【Bitmap】【Color】
using System.Drawing.Imaging;         //【BitmapData】
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices; // [DllImport("ftd2xx.dll")]参照用のusing指定

namespace FTDI_BitBangI2C_3
{
    public partial class Form1 : Form
    {
        private UInt32 FT_Handle1; //【ハンドル番号】
        private UInt32 FT_Status;  //【ステータス】
        private int RH, RL, RD;
        private double AccX, AccY, AccZ;
        private int Count = 0;
        private byte[] TxBuf = new byte[512]; //【Sequence1or4による送信用】
        private byte[] RxBuf = new byte[512];
        // D7:INT, D4:SCL, D1:RxSDA, D0:TxSDA
        //【SCL,SDAのデータは正論理で書いておいて論理をひっくり返す】
        //●指定レジスタに指定データを書き込む
        private byte[] Sequence1 = {
             //①【Start Condition】4bytes
             0x11, 0x11, 0x10, 0x00,
             //②【スレーブ・アドレス】28bytes SA6==>SA0 [0001111B]
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //③【アクセス方向】4bytes RW 0:Write 1:Read
             0x00, 0x10, 0x10, 0x00, // ■レジスタへの書込みなのでLow固定
             //④【スレーブからのAck】4bytes
             0x01, 0x11, 0x11, 0x01, // ■スレーブからAckを受信　High固定
             //⑤【レジスタアドレス】32bytes　RA7==>RA0 [00000000B] ★レジスタアドレスをORする
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00,
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00,
             //⑥【スレーブからのAck】4bytes
             0x01, 0x11, 0x11, 0x01, // ■スレーブからAckを受信　High固定
             //⑦【設定データ】32bytes　D7==>D0 [00000000B] ★設定データをORする
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00,
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00,
             //⑧【スレーブからのAck】4bytes
             0x01, 0x11, 0x11, 0x01, // ■スレーブからAckを受信　High固定
             //⑨【Stop Condition】4bytes
             0x00, 0x10, 0x11, 0x11 };
        //●指定レジスタからデータを読み出す
        private byte[] Sequence4 = {
             //①【Start Condition】4bytes
             0x11, 0x11, 0x10, 0x00,
             //②【スレーブ・アドレス】28bytes SA6==>SA0 [0001111B]
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //③【アクセス方向】4bytes RW 0:Write 1:Read
             0x00, 0x10, 0x10, 0x00, // ■レジスタへの書込みなのでLow固定
             //④【スレーブからのAck】4bytes
             0x01, 0x11, 0x11, 0x01, // ■スレーブからAckを受信　High固定
             //⑤【レジスタアドレス】32bytes　RA7==>RA0 [00000000B] ★レジスタアドレスをORする
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00,
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00,
             //⑥【スレーブからのAck】4bytes
             0x01, 0x11, 0x11, 0x01, // ■スレーブからAckを受信　High固定
             //⑦【Re-Start Condition】4bytes
             0x01, 0x11, 0x10, 0x00,
             //⑧【スレーブ・アドレス】28bytes SA6==>SA0 [0001111B]
             0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x00, 0x10, 0x10, 0x00, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑨【アクセス方向】4bytes RW 0:Write 1:Read
             0x01, 0x11, 0x11, 0x01, // ■レジスタからの読出しなのでHigh固定
             //⑩【スレーブからのAck】4bytes
             0x01, 0x11, 0x11, 0x01, // ■スレーブからAckを受信　High固定
             //⑪【読出しデータ】32bytes　D7==>D0 [11111111B]
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑫【スレーブからのAck】4bytes
             0x00, 0x10, 0x10, 0x00, // ■マスターからAckを返信　Low固定
             //⑪【読出しデータ】32bytes　D7==>D0 [11111111B]
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑫【スレーブからのAck】4bytes
             0x00, 0x10, 0x10, 0x00, // ■マスターからAckを返信　Low固定
             //⑪【読出しデータ】32bytes　D7==>D0 [11111111B]
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑫【スレーブからのAck】4bytes
             0x00, 0x10, 0x10, 0x00, // ■マスターからAckを返信　Low固定
             //⑪【読出しデータ】32bytes　D7==>D0 [11111111B]
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑫【スレーブからのAck】4bytes
             0x00, 0x10, 0x10, 0x00, // ■マスターからAckを返信　Low固定
             //⑪【読出しデータ】32bytes　D7==>D0 [11111111B]
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑫【スレーブからのAck】4bytes
             0x00, 0x10, 0x10, 0x00, // ■マスターからAckを返信　Low固定
             //⑪【読出しデータ】32bytes　D7==>D0 [11111111B]
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01, 0x01, 0x11, 0x11, 0x01,
             //⑫【スレーブからのNack】4bytes
             0x01, 0x11, 0x11, 0x01, // ■マスターからNackを返信　High固定
             //⑬【Stop Condition】4bytes
             0x00, 0x10, 0x11, 0x11 };

        private float HandSensor1;                //【加速度センサで測定された値の絶対値】
        private int pFIFO = 0;                    //
        private const int nFIFO = 600;            //
        private float[] FIFOa = new float[nFIFO]; //
        private float[] FIFOx = new float[nFIFO]; //
        private float[] FIFOy = new float[nFIFO]; //
        private float[] FIFOz = new float[nFIFO]; //
        private Bitmap DSP_bmp1;                   //【ColorImage表示用Bitmapオブジェクト】
        private Rectangle DSP_rect1;               //【ColorImage表示枠】

        private Boolean flagRecord = false;
        private string Log = "";
        private int TimeCount;

        //**********************************************************************
        //【メインForm】
        //**********************************************************************
        public Form1()
        {
            InitializeComponent();                       //【Form1の初期化】
            int n = CheckKionixSensors();                //【接続されているセンサの個数を確認する】
            if (n > 0) FT_Handle1 = InitKionixSensor(0); //【0番目のKionixセンサの初期化(オープン)を行う】
            if (FT_Handle1 > 0)                          //【Kionixセンサがオープンできた場合】
            {
                timer1.Tick += new EventHandler(timer1_Tick); //●timerイベント関数をtimer1_Tickと設定する
                timer1.Interval = 1;                          //●timer1をスレッドの代用とする(1mS間隔はWindowsの制約であり得ず実際は20mS程度)
                timer1.Enabled = true;                        //●timerイベントを許可する
            }
            for (int i = 0; i < nFIFO; i++) { FIFOa[i] = FIFOx[i] = FIFOy[i] = FIFOz[i] = 205f; } //【FIFOを0クリア】

            DSP_bmp1 = new Bitmap(nFIFO, 410, System.Drawing.Imaging.PixelFormat.Format32bppRgb); //【ColorImage表示用】
            DSP_rect1 = new Rectangle(0, 0, nFIFO, 410);                                          //【ColorImage用枠】
            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing); //■終了イベント関数をForm1_FormClosingと設定する
            checkBox1.CheckedChanged += new EventHandler(checkBox1_CheckedChanged);
        }

        void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked)
            {
                flagRecord = true;
                Log = "No.,TimeStamp, X,Y,Z,ABS\r\n";
            }
            else
            {
                flagRecord = false;
                saveFileDialog1.Title = "保存するデータに名前を付けます";
                saveFileDialog1.ShowDialog();
                string fp = saveFileDialog1.FileName;
                if ( fp != "") { WriteFile(fp, Log); }
            }
        }

        //**********************************************************************
        //【終了処理】
        //**********************************************************************
        void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (FT_Handle1 > 0) { FT_Status = FT_Close(FT_Handle1); } //【FT232Rを閉じる】
        }

        //**********************************************************************
        //【タイマーイベント】
        //**********************************************************************
        unsafe void timer1_Tick(object sender, EventArgs e)
        {   //【timerイベントで加速度センサの測定値を読込、表示を行う】
            SendReadCommand(FT_Handle1);
            Read6Bytes(FT_Handle1, out AccX, out AccY, out AccZ); //①【Sequence4を使ってAccX, AccY, AccZを読み出す】
            HandSensor1 = (float)Math.Sqrt(AccX * AccX + AccY * AccY + AccZ * AccZ); //【絶対値を求める】
            FIFOx[pFIFO] = 410.0f - (float)AccX * 100.0f - 205.0f; //【FIFOに格納する】
            FIFOy[pFIFO] = 410.0f - (float)AccY * 100.0f - 205.0f; //  : (1gを100画素に割り当てている)
            FIFOz[pFIFO] = 410.0f - (float)AccZ * 100.0f - 205.0f; //  :
            FIFOa[pFIFO] = 410.0f - HandSensor1 * 100.0f - 205.0f; //  :
            pFIFO++; if (pFIFO == nFIFO) pFIFO = 0;                //【FIFOの格納ポインタの更新】

            if (flagRecord)
            {
                Log += Count.ToString() + "," + System.Environment.TickCount.ToString() + "," + AccX.ToString() + "," + AccY.ToString() + "," + AccZ.ToString() + "," + HandSensor1.ToString() + "\r\n";
            }

            label1.Text = "Xout=" + AccX.ToString() + "[g]";            //②Xoutを表示する
            label2.Text = "Yout=" + AccY.ToString() + "[g]";            //③Youtを表示する
            label3.Text = "Zout=" + AccZ.ToString() + "[g]";            //④Zoutを表示する
            label4.Text = "|加速度|=" + HandSensor1.ToString() + "[g]"; //⑤絶対値を求め表示する
            label5.Text = "読出し回数=" + Count.ToString();             //⑥読出し回数(Count)を表示する
            Count++;                                                    //⑦Countをインクリメント(+1)

            pictureBox1.Image = DSP_bmp1;
            Graphics g = Graphics.FromImage(pictureBox1.Image);
            g.FillRectangle(new SolidBrush(Color.PowderBlue), DSP_rect1);    //【画面クリア】
            g.DrawRectangle(new Pen(Color.Yellow, 3.0f), 0, 0, nFIFO, 410);  //【枠線】
            g.DrawLine(new Pen(Color.Yellow, 3.0f), 0, 5, nFIFO - 1, 5);     //【+2gの横線】
            g.DrawLine(new Pen(Color.Yellow, 3.0f), 0, 105, nFIFO - 1, 105); //【+1gの横線】
            g.DrawLine(new Pen(Color.Yellow, 3.0f), 0, 205, nFIFO - 1, 205); //【0gの横線】
            g.DrawLine(new Pen(Color.Yellow, 3.0f), 0, 305, nFIFO - 1, 305); //【-1gの横線】
            g.DrawLine(new Pen(Color.Yellow, 3.0f), 0, 405, nFIFO - 1, 405); //【-2gの横線】
            g.DrawLine(new Pen(Color.Yellow, 3.0f), (int)(nFIFO / 2), 0, (int)(nFIFO / 2), 409); //【中央縦線】

            PointF[] Pabs = new PointF[nFIFO]; //【波形描画用配列】絶対値
            PointF[] Px = new PointF[nFIFO];   //【波形描画用配列】x成分
            PointF[] Py = new PointF[nFIFO];   //【波形描画用配列】y成分
            PointF[] Pz = new PointF[nFIFO];   //【波形描画用配列】z成分
            for (int i = 0; i < nFIFO; i++) //【FIFOに格納されたデータを時刻順に波形描画用配列に格納】
            {
                Pabs[i] = new PointF((float)i, FIFOa[pFIFO]);
                Px[i] = new PointF((float)i, FIFOx[pFIFO]);
                Py[i] = new PointF((float)i, FIFOy[pFIFO]);
                Pz[i] = new PointF((float)i, FIFOz[pFIFO]);
                pFIFO++; if (pFIFO == nFIFO) { pFIFO = 0; }
            }
            g.DrawLines(new Pen(Color.Red, 3.0f), Pabs);  //【加速度の絶対値】
            g.DrawLines(new Pen(Color.Orange, 3.0f), Px); //【加速度のx成分】
            g.DrawLines(new Pen(Color.Blue, 3.0f), Py);   //【加速度のy成分】
            g.DrawLines(new Pen(Color.Green, 3.0f), Pz);  //【加速度のz成分】
            g.Dispose();     //【グラフィックスを開放】
            this.Validate(); //【変更を更新】
        }

        //**********************************************************************
        //【InitKionixSensor】デバイス番号0を初期化し、ハンドル番号を返す
        //**********************************************************************
        unsafe private short CheckKionixSensors()
        {
            UInt32 numDevices; //【FTDI社のデバイス総数】
            FT_Status = FT_ListDevices(&numDevices, null, 0x80000000); //【デバイス数を調べる】
            return (short)numDevices; //【センサ数を返す】
        }

        //**********************************************************************
        //【InitKionixSensor】指定デバイス番号を初期化し、ハンドル番号を返す
        //**********************************************************************
        unsafe private UInt32 InitKionixSensor(short num)
        {
            UInt32 FT_Handle;  //【ハンドル番号】local変数

            FT_Status = FT_Open(num, &FT_Handle);             //【デバイス番号0をオープン】センサ個数は1を前提として記述しています
            FT_Status = FT_SetBitMode(FT_Handle, 0x7D, 0x00); //【bit7とbit1が入力】【Reset】
            FT_Status = FT_SetBaudRate(FT_Handle, 115200);    //【9600bps】19200, 38400, 115200, 921600
            FT_Status = FT_SetBitMode(FT_Handle, 0x7D, 0x04); //【bit7とbit1が入力】【同期BitBang】
            SetWriteData(0x1B, 0x50); //①レジスタ[1BH] Stand Byモードで12bit分解能、±8gに設定
            WriteOneByte(FT_Handle);  //【Sequence1で送信】
            System.Threading.Thread.Sleep(1000); //【1秒間待機する】
            WriteOneByte(FT_Handle);  //【Sequence1で送信】
            SetWriteData(0x1B, 0xD0); //②レジスタ[1BH] Operatingモードに切り替える
            WriteOneByte(FT_Handle);  //【Sequence1で送信】
            DisplayTxRxBuf();         //【同期BitBangモードで出力された送信バッファと受信バッファの内容をバイナリで表示する】
            SetReadData(0x06);        //【Sequence4の読み出し開始レジスタを設定する】
            return FT_Handle;         //【センサ接続】
        }

        //**********************************************************************
        //【WriteOneByte】Sequence1で指定レジスタに書き込むデータをバッファTxBufS1WRにセットする
        //**********************************************************************
        unsafe private void SetWriteData(byte RegAddress, byte WriteData)
        {
            uint i;
            for (i = 0; i < Sequence1.Length; i++) { TxBuf[i] = Sequence1[i]; } //【Sequence1を送信バッファにコピー】
            SetTxData(40, RegAddress);  //【追記】【レジスタアドレスを設定】
            SetTxData(76, WriteData);   //【追記】【書込みデータを設定】 
        }

        //**********************************************************************
        //【WriteOneByte】Sequence1で指定ハンドル番号のデバイスにデータを送信する
        //**********************************************************************
        unsafe private void WriteOneByte(UInt32 FT_Handle)
        {
            uint i;
            uint writeSize, readSize;
            i = (uint)Sequence1.Length; //【送信バイト数を取得⇒i】
            FT_Status = FT_Write(FT_Handle, TxBuf, i, &writeSize); //【同期BitBangでTxBufを送信】
            FT_Status = FT_Read(FT_Handle, RxBuf, i, &readSize);       //【同期BitBangで受信したデータをRxBufに格納】
        }

        //**********************************************************************
        //【SetReadData】Sequence4でマルチバイトリードを開始するレジスタを指定する値を書き込む
        //**********************************************************************
        unsafe private void SetReadData(byte RegAddress)
        {
            uint i;
            for (i = 0; i < Sequence4.Length; i++) { TxBuf[i] = Sequence4[i]; } //【Sequence3を送信バッファにコピー】
            SetTxData(40, RegAddress);  //【追記】【レジスタアドレスを設定】
        }

        //**********************************************************************
        //【SendReadCommand】Sequence4で指定ハンドル番号のデバイスの指定レジスタから6バイトを読み込むコマンドを送出
        //**********************************************************************
        unsafe private void SendReadCommand(UInt32 FT_Handle)
        {
            uint i;
            uint writeSize;
            i = (uint)Sequence4.Length; //【送信バイト数を取得⇒i】
            FT_Status = FT_Write(FT_Handle, TxBuf, i, &writeSize); //【同期BitBangでTxBufを送信】
        }

        //**********************************************************************
        //【Read6Bytes】Sequence4で指定ハンドル番号のデバイスの指定レジスタから6バイトを読み込む【マルチバイトリード】
        //**********************************************************************
        unsafe private void Read6Bytes(UInt32 FT_Handle, out double X, out double Y, out double Z)
        {
            uint i;
            uint readSize;
            const double keisu = 1.0 / 4096.0; //±8g 
            i = (uint)Sequence4.Length; //【送信バイト数を取得⇒i】
            FT_Status = FT_Read(FT_Handle, RxBuf, i, &readSize);   //【同期BitBangで受信したデータをRxBufに格納】
            RL = ReadOneByte(116);          //【受信バッファからレジスタXoutLを取得する】
            RH = ReadOneByte(116 + 36);     //【受信バッファからレジスタXoutHを取得する】
            RD = RH * 256 + RL; if (RD > 32767) RD = RD - 65536; //【２の補数形式に変換】
            X = RD * keisu; //【結果をXに格納する】
            RL = ReadOneByte(116 + 36 * 2); //【受信バッファからレジスタYoutLを取得する】
            RH = ReadOneByte(116 + 36 * 3); //【受信バッファからレジスタYoutHを取得する】
            RD = RH * 256 + RL; if (RD > 32767) RD = RD - 65536; //【２の補数形式に変換】
            Y = RD * keisu; //【結果をYに格納する】
            RL = ReadOneByte(116 + 36 * 4); //【受信バッファからレジスタZoutLを取得する】
            RH = ReadOneByte(116 + 36 * 5); //【受信バッファからレジスタZoutHを取得する】
            RD = RH * 256 + RL; if (RD > 32767) RD = RD - 65536; //【２の補数形式に変換】
            Z = RD * keisu; //【結果をZに格納する】
        }

        //**********************************************************************
        //【ReadOneByte】受信バッファRxBufの指定オフセット位置から１バイト分の情報を取得する
        //**********************************************************************
        private byte ReadOneByte(int ofs)
        {   //
            uint i,j,k;
            j = 0; k = 0x80;
            for (i = 0; i < 8; i++)
            {
                if ((RxBuf[ofs + 3 + 4 * i] & 0x02) > 0) j = j | k;
                k = k >> 1;
            }
            return (byte)j;
        }

        //**********************************************************************
        //【SetTxData】送信バッファTxBufの指定オフセット位置に１バイト分の指定情報を追記する
        //**********************************************************************
        unsafe private void SetTxData(int Offset, byte Data)
        {
            int i;
            byte reg, bit;

            reg = Data;
            for (i = 0; i < 8; i++)
            {
                if ((reg & 0x80) > 0) { bit = 1; } else { bit = 0; }
                TxBuf[Offset + 4 * i] |= bit;
                TxBuf[Offset + 4 * i + 1] |= bit;
                TxBuf[Offset + 4 * i + 2] |= bit;
                TxBuf[Offset + 4 * i + 3] |= bit;
                reg = (byte)((reg & 0x7F) << 1);
            }
        }

        //*******************************************
        //【FTDI社D2XXドライバーDLL参照定義】
        //*******************************************
        //【FT Open】
        [DllImport("ftd2xx.dll")] unsafe private static extern UInt32 FT_Open(Int16 DeviceNumber, UInt32* ftHandle);
        //【FT Close】
        [DllImport("ftd2xx.dll")] unsafe private static extern UInt32 FT_Close(UInt32 ftHandle);
        //【FT ListDevices】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_ListDevices(void* pArg1, void* pArg2, UInt32 Flags);
        //【FT SetBitMode】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_SetBitMode(UInt32 ftHandle, Byte Mask, Byte Enable);
        // Mask 1:出力 0:入力　Enable: 0x4: 同期Bit Bangモード 1:BitBangモード 0:ハンドシェークモード
        //【FT GetBitMode】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_GetBitMode(UInt32 ftHandle, [MarshalAs(UnmanagedType.LPArray)] byte[] bdata);
        //【FT SetBaudRate】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_SetBaudRate(UInt32 ftHandle, UInt32 BaudRate);
        //【FT Read】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_Read(UInt32 ftHandle, [MarshalAs(UnmanagedType.LPArray)] byte[] bdata, UInt32 dwBytesToRead, UInt32* lpBytesReturned);
        //【FT Write】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_Write(UInt32 ftHandle, [MarshalAs(UnmanagedType.LPArray)] byte[] bdata, UInt32 dwBytesToWrite, UInt32* lpBytesWritten);
        //【FT Purge】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_Purge(UInt32 ftHandle, UInt32 Mask);
        //【FT GetStatus】
        [DllImport("ftd2xx.dll")]
        unsafe private static extern UInt32 FT_GetStatus(UInt32 ftHandle, UInt16* lpRxBytes, UInt16* lpTxBytes, UInt16* lpEventWord);


        //*******************************************
        //【Debug用】【指定バイトデータDataをテキストバイナリ形式に変換する】
        //*******************************************
        private string ConvertBinary(byte Data)
        {
            int j = 0x80;
            string s = "";
            for (int i = 0; i < 8; i++)
            {
                if ((Data & j) > 0) { s += "1"; } else { s += "0"; }
                j = j >> 1;
            }
            return (s+"B");
        }

        //*******************************************
        //【Debug用】【同期BitBangモードで送出/受信した情報をバイナリ形式でtextBox1に展開する】
        //*******************************************
        private void DisplayTxRxBuf()
        {   
            for (int i = 0; i < Sequence1.Length; i++)
            {
                textBox1.Text += ConvertBinary(TxBuf[i]) + "   ";
                textBox1.Text += ConvertBinary(RxBuf[i]) + "\r\n";
            }
            textBox1.Text += "\r\n";
        }

        //*******************************************
        //【データ保存用】
        //*******************************************
        private void WriteFile(string FilePath, string txt)
        {
            Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
            StreamWriter writer =
              new StreamWriter(FilePath, true, sjisEnc);
            writer.Write(txt);
            writer.Close();
        }

    }
}
